Package: tok
Title: Fast Text Tokenization
Version: 0.2.2
Authors@R: c(
    person("Tomasz", "Kalinowski", , "tomasz@posit.co", c("ctb", "cre")),
    person("Daniel", "Falbel", , "dfalbel@gmail.com", c("aut")),
    person("Regouby", "Christophe", , "christophe.regouby@free.fr", c("ctb")),
    person(family = "Posit", role = c("cph"))
  )
Description: 
  Interfaces with the 'Hugging Face' tokenizers library to provide implementations
  of today's most used tokenizers such as the 'Byte-Pair Encoding' algorithm 
  <https://huggingface.co/docs/tokenizers/index>. It's extremely fast for both 
  training new vocabularies and tokenizing texts.
License: MIT + file LICENSE
SystemRequirements: Cargo (Rust's package manager), rustc >= 1.91
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.2.0)
Imports: R6, cli
Suggests: rmarkdown, testthat (>= 3.0.0), hfhub (>= 0.1.1), withr
Config/testthat/edition: 3
URL: https://github.com/mlverse/tok
BugReports: https://github.com/mlverse/tok/issues
Config/rextendr/version: 0.5.0
NeedsCompilation: yes
Packaged: 2026-04-21 23:57:56 UTC; dfalbel
Author: Tomasz Kalinowski [ctb, cre],
  Daniel Falbel [aut],
  Regouby Christophe [ctb],
  Posit [cph]
Maintainer: Tomasz Kalinowski <tomasz@posit.co>
Repository: CRAN
Date/Publication: 2026-04-22 14:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 22:53:35 UTC; unix
Archs: tok.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("$",RDecoder)
S3method("$",RDecoderByteLevel)
S3method("$",REncoding)
S3method("$",RModel)
S3method("$",RModelBPE)
S3method("$",RModelUnigram)
S3method("$",RModelWordPiece)
S3method("$",RNormalizer)
S3method("$",RNormalizerNFC)
S3method("$",RNormalizerNFKC)
S3method("$",RPostProcessor)
S3method("$",RPostProcessorByteLevel)
S3method("$",RPreTokenizer)
S3method("$",RPreTokenizerByteLevel)
S3method("$",RPreTokenizerWhitespace)
S3method("$",RTokenizer)
S3method("$",RTrainer)
S3method("$",RTrainerBPE)
S3method("$",RTrainerUnigram)
S3method("$",RTrainerWordPiece)
S3method("[[",RDecoder)
S3method("[[",RDecoderByteLevel)
S3method("[[",REncoding)
S3method("[[",RModel)
S3method("[[",RModelBPE)
S3method("[[",RModelUnigram)
S3method("[[",RModelWordPiece)
S3method("[[",RNormalizer)
S3method("[[",RNormalizerNFC)
S3method("[[",RNormalizerNFKC)
S3method("[[",RPostProcessor)
S3method("[[",RPostProcessorByteLevel)
S3method("[[",RPreTokenizer)
S3method("[[",RPreTokenizerByteLevel)
S3method("[[",RPreTokenizerWhitespace)
S3method("[[",RTokenizer)
S3method("[[",RTrainer)
S3method("[[",RTrainerBPE)
S3method("[[",RTrainerUnigram)
S3method("[[",RTrainerWordPiece)
export(decoder_byte_level)
export(encoding)
export(model_bpe)
export(model_unigram)
export(model_wordpiece)
export(normalizer_nfc)
export(normalizer_nfkc)
export(pre_tokenizer)
export(pre_tokenizer_byte_level)
export(pre_tokenizer_whitespace)
export(processor_byte_level)
export(tok_decoder)
export(tok_model)
export(tok_normalizer)
export(tok_processor)
export(tok_trainer)
export(tokenizer)
export(trainer_bpe)
export(trainer_unigram)
export(trainer_wordpiece)
importFrom(R6,R6Class)
importFrom(cli,cli_abort)
useDynLib(tok, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# tok

<!-- badges: start -->

[![R build
status](https://github.com/mlverse/tok/workflows/R-CMD-check/badge.svg)](https://github.com/mlverse/tok/actions)
[![CRAN
status](https://www.r-pkg.org/badges/version/tok)](https://CRAN.R-project.org/package=tok)
[![](https://cranlogs.r-pkg.org/badges/tok)](https://cran.r-project.org/package=tok)
[![extendr](https://img.shields.io/badge/extendr-%5E0.9.0-276DC2)](https://extendr.rs/extendr/extendr_api/)
<!-- badges: end -->

tok provides bindings to the
[🤗tokenizers](https://huggingface.co/docs/tokenizers/v0.13.3/en/index)
library. It uses the same Rust libraries that powers the Python
implementation.

We still don’t provide the full API of tokenizers. Please open a issue
if there’s a feature you are missing.

## Installation

You can install tok from CRAN using:

    install.packages("tok")

Installing tok from source requires working Rust toolchain. We recommend
using [rustup.](https://rustup.rs/)

On Windows, you’ll also have to add the `i686-pc-windows-gnu` and
`x86_64-pc-windows-gnu` targets:

    rustup target add x86_64-pc-windows-gnu
    rustup target add i686-pc-windows-gnu

Once Rust is working, you can install this package via:

``` r
remotes::install_github("dfalbel/tok")
```

## Features

We still don’t have complete support for the 🤗tokenizers API. Please
open an issue if you need a feature that is currently not implemented.

## Loading tokenizers

`tok` can be used to load and use tokenizers that have been previously
serialized. For example, HuggingFace model weights are usually
accompanied by a ‘tokenizer.json’ file that can be loaded with this
library.

To load a pre-trained tokenizer from a json file, use:

``` r
path <- testthat::test_path("assets/tokenizer.json")
tok <- tok::tokenizer$from_file(path)
```

Use the `encode` method to tokenize sentendes and `decode` to transform
them back.

``` r
enc <- tok$encode("hello world")
tok$decode(enc$ids)
#> [1] "hello world"
```

## Using pre-trained tokenizers

You can also load any tokenizer available in HuggingFace hub by using
the `from_pretrained` static method. For example, let’s load the GPT2
tokenizer with:

``` r
tok <- tok::tokenizer$from_pretrained("gpt2")
enc <- tok$encode("hello world")
tok$decode(enc$ids)
#> [1] "hello world"
```

# From https://github.com/yutannihilation/string2path/blob/main/update_authors.R
# Copyright Hiroaki Yutani
# License: MIT

rlang::check_installed("RcppTOML")
rlang::check_installed("stringr")

library(RcppTOML)

## Update inst/AUTHORS

VENDOR_PATH <- "src/rust/vendor"
manifests <- list.files(VENDOR_PATH, pattern = "Cargo.toml", recursive = TRUE)

l <- lapply(manifests, \(x) RcppTOML::parseTOML(file.path(VENDOR_PATH, x))$package)

names <- vapply(l, \(x) x[["name"]], FUN.VALUE = character(1L))
versions <- vapply(l, \(x) x[["version"]], FUN.VALUE = character(1L))

authors <- vapply(l, \(x) {
  # Remove email addresses
  authors <- stringr::str_remove(x[["authors"]], "\\s+<.+>")
  paste(authors, collapse = ", ")
}, FUN.VALUE = character(1L))

licenses <- vapply(l, \(x) x[["license"]], FUN.VALUE = character(1L))

dir.create("inst", showWarnings = FALSE)

cat("The authors of the dependency Rust crates:

", file = "inst/AUTHORS")

authors_flattened <- vapply(stringr::str_split(authors, ",\\s+"), \(x) {
  paste(x, collapse = "\n  ")
}, FUN.VALUE = character(1L))

cat(paste(
  names, " (version ", versions, "):\n  ",
  authors_flattened,
  "\n",
  sep = "",
  collapse = "\n"
), file = "inst/AUTHORS", append = TRUE)

## Update LICENSE.note

cat("This package contains the Rust source code of the dependencies in src/rust/vendor.tar.xz
The authorships and the licenses are listed below. In summary, all libraries are
distributed either under the MIT license or under MIT/Apache-2.0 dual license [1].

Note that, when Cargo (Rust’s build system and package manager) is not installed
on the machine, the pre-compiled binary will be downloaded on building this
package. The binary is compiled using the same Rust code, so the authorships and
the licenses are the same as listed here.

[1]: The unicode-indent library shows 'Unicode-DFS-2016' license because it
     contains some test data generated by using the Unicode Character Database.
     So, this license is not applied to the actual sources that get compiled.
     Please refer to the License section of the library's README
     (https://crates.io/crates/unicode-ident) for the details.

===============================

", file = "LICENSE.note")

cat(paste(
  "Name:    ", names,    "\n",
  "Files:   vendor/", names,    "/*\n",
  "Authors: ", authors,  "\n",
  "License: ", licenses, "\n",
  sep = "",
  collapse = "\n------------------------------\n\n"
), file = "LICENSE.note", append = TRUE)

#' Generic class for tokenization models
#' @export
#' @family model
tok_model <- R6::R6Class(
  "tok_model",
  public = list(
    #' @field .model stores the pointer to the model. internal
    .model = NULL,
    #' @description
    #' Initializes a genric abstract tokenizer model
    #' @param model Pointer to a tokenization model
    initialize = function(model) {
      self$.model <- RModel$new(model)
    }
  )
)

#' BPE model
#' @export
#' @family model
model_bpe <- R6::R6Class(
  "tok_model_bpe",
  inherit = tok_model,
  public = list(
    #' @description Initializes a BPE model
    #' An implementation of the BPE (Byte-Pair Encoding) algorithm
    #' @param vocab A named integer vector of string keys and their corresponding ids. Default: `NULL`
    #' @param merges A list of pairs of tokens (`[character, character]`). Default: `NULL`.
    #' @param cache_capacity The number of words that the BPE cache can contain.
    #'        The cache speeds up the process by storing merge operation results. Default: `NULL.`
    #' @param dropout A float between 0 and 1 representing the BPE dropout to use. Default: `NULL`
    #' @param unk_token The unknown token to be used by the model. Default: `NULL```.
    #' @param continuing_subword_prefix The prefix to attach to subword units that don’t
    #'        represent the beginning of a word. Default: `NULL`
    #' @param end_of_word_suffix The suffix to attach to subword units that represent
    #'        the end of a word. Default: `NULL`
    #' @param fuse_unk Whether to fuse any subsequent unknown tokens into a single one. Default: `NULL`.
    #' @param byte_fallback Whether to use the spm byte-fallback trick. Default: `FALSE`.
    initialize = function(
    vocab = NULL, merges = NULL, cache_capacity = NULL, 
    dropout = NULL, unk_token = NULL, continuing_subword_prefix = NULL,
    end_of_word_suffix = NULL, fuse_unk = NULL, byte_fallback = FALSE
    ) {
      super$initialize(RModelBPE$new(
        vocab = vocab,
        merges = merges, 
        cache_capacity = cache_capacity,
        dropout = dropout,
        unk_token = unk_token,
        continuing_subword_prefix = continuing_subword_prefix,
        end_of_word_suffix = end_of_word_suffix,
        fuse_unk = fuse_unk,
        byte_fallback = byte_fallback
      ))
    }
  )
)

#' An implementation of the WordPiece algorithm
#' 
#' @family model
#' @export
model_wordpiece <- R6::R6Class(
  "tok_model_wordpiece",
  inherit = tok_model,
  public = list(
    #' @description
    #' Constructor for the wordpiece tokenizer
    #' @param vocab A dictionary of string keys and their corresponding ids.
    #'        Default: `NULL`.
    #' @param unk_token The unknown token to be used by the model.
    #'        Default: `NULL`.
    #' @param max_input_chars_per_word The maximum number of characters to allow in a single word.
    #'        Default: `NULL`.
    initialize = function(vocab = NULL, unk_token = NULL, max_input_chars_per_word = NULL) {
      super$initialize(RModelWordPiece$new(
        vocab = vocab,
        unk_token = unk_token,
        max_input_chars_per_word = max_input_chars_per_word
      ))
    }
  )
)

#' An implementation of the Unigram algorithm
#' @export
#' @family model
model_unigram <- R6::R6Class(
  "tok_model_unigram",
  inherit = tok_model,
  public = list(
    #' @description
    #' Constructor for Unigram Model
    #' @param vocab A dictionary of string keys and their corresponding relative score.
    #'        Default: `NULL`.
    #' @param unk_id The unknown token id to be used by the model.
    #'        Default: `NULL`.
    #' @param byte_fallback Whether to use byte-fallback trick. Default: `FALSE`.
    initialize = function(vocab = NULL, unk_id = NULL, byte_fallback = FALSE) {
      super$initialize(RModelUnigram$new(
        vocab = vocab,
        unk_id = unk_id,
        byte_fallback = byte_fallback
      ))
    }
  )
)

#' Tokenizer
#' @description 
#' A Tokenizer works as a pipeline. It processes some raw text as input and outputs 
#' an [encoding].
#' 
#' @importFrom R6 R6Class
#' @importFrom cli cli_abort
#' 
#' @examples
#' withr::with_envvar(c(HUGGINGFACE_HUB_CACHE = tempdir()), {
#' try({
#' tok <- tokenizer$from_pretrained("gpt2")
#' tok$encode("Hello world")$ids
#' })
#' })
#' 
#' @returns
#' A tokenizer that can be used for encoding character strings or decoding
#' integers.
#' 
#' @export
tokenizer <- R6::R6Class(
  classname = "tok_tokenizer",
  public = list(
    #' @field .tokenizer (unsafe usage) Lower level pointer to tokenizer
    .tokenizer = NULL,
    
    #' @description Initializes a tokenizer
    #' @param tokenizer Will be cloned to initialize a new tokenizer
    initialize = function(tokenizer) {
      if (inherits(tokenizer, "RTokenizer")) {
        self$.tokenizer <- tokenizer
      } else if (inherits(tokenizer, "tok_model")) {
        self$.tokenizer <- RTokenizer$from_model(tokenizer$.model)
      } else {
        self$.tokenizer <- RTokenizer$new(tokenizer$.tokenizer)
      }
    },
    
    #' @description 
    #' Encode the given sequence and pair. This method can process raw text sequences
    #' as well as already pre-tokenized sequences.
    #' @param sequence The main input sequence we want to encode. This sequence can 
    #'  be either raw text or pre-tokenized, according to the is_pretokenized argument
    #' @param pair An optional input sequence. The expected format is the same 
    #'  that for sequence.
    #' @param is_pretokenized Whether the input is already pre-tokenized
    #' @param add_special_tokens Whether to add the special tokens
    encode = function(sequence, pair = NULL, is_pretokenized = FALSE, add_special_tokens = TRUE) {
      self$.tokenizer$encode(sequence, pair, is_pretokenized, add_special_tokens)
    },
    
    #' @description 
    #' Decode the given list of ids back to a string
    #' @param ids The list of ids that we want to decode
    #' @param skip_special_tokens Whether the special tokens should be removed from the decoded string
    decode = function(ids, skip_special_tokens = TRUE) {
      self$.tokenizer$decode(ids, skip_special_tokens)
    },
    
    #' @description 
    #' Encodes a batch of sequences. Returns a list of [encoding]s.
    #' @param input A list of single sequences or pair sequences to encode. Each 
    #'  sequence can be either raw text or pre-tokenized, according to the is_pretokenized 
    #'  argument.
    #' @param is_pretokenized Whether the input is already pre-tokenized
    #' @param add_special_tokens Whether to add the special tokens
    encode_batch = function(input, is_pretokenized = FALSE, add_special_tokens = TRUE) {
      self$.tokenizer$encode_batch(input, is_pretokenized, add_special_tokens)
    },
    
    #' @description 
    #' Decode a batch of ids back to their corresponding string
    #' @param sequences The batch of sequences we want to decode
    #' @param skip_special_tokens Whether the special tokens should be removed from the decoded strings
    decode_batch = function(sequences, skip_special_tokens = TRUE) {
      self$.tokenizer$decode_batch(sequences, skip_special_tokens)
    },
    
    #' @description 
    #' Creates a tokenizer from the path of a serialized tokenizer.
    #' This is a static method and should be called instead of `$new` when initializing
    #' the tokenizer.
    #' @param path Path to tokenizer.json file
    from_file = function(path) {
      cli::cli_abort(gettext("This is a static method. Not available for tokenizers instances."))
    },
    
    #' @description 
    #' Instantiate a new Tokenizer from an existing file on the Hugging Face Hub.
    #' @param identifier The identifier of a Model on the Hugging Face Hub, that 
    #'    contains a tokenizer.json file
    #' @param revision  A branch or commit id
    #' @param auth_token An optional auth token used to access private repositories 
    #'    on the Hugging Face Hub
    from_pretrained = function(identifier, revision = "main", auth_token = NULL) {
      cli::cli_abort(gettext("This is a static method. Not available for tokenizers instances."))
    },
    
    #' @description
    #' Train the Tokenizer using the given files.
    #' Reads the files line by line, while keeping all the whitespace, even new lines. 
    #' @param trainer an instance of a trainer object, specific to that tokenizer type.
    #' @param files character vector of file paths.
    train = function(files, trainer) {
      if (!inherits(trainer, "tok_trainer"))
        cli::cli_abort(gettext("{.arg trainer} must inherit from {.cls tok_trainer}."))
      
      self$.tokenizer$train_from_files(trainer$.trainer, normalizePath(files))
    },
    
    #' @description 
    #' Train the tokenizer on a chracter vector of texts
    #' @param texts a character vector of texts.
    #' @param trainer an instance of a trainer object, specific to that tokenizer type.
    train_from_memory = function(texts, trainer) {
      self$.tokenizer$train_from_sequences(trainer$.trainer, texts)
    },
    
    #' @description
    #' Saves the tokenizer to a json file
    #' @param path  A path to a file in which to save the serialized tokenizer.
    #' @param pretty Whether the JSON file should be pretty formatted.
    save = function(path, pretty = TRUE) {
      self$.tokenizer$save(normalizePath(path, mustWork = FALSE), pretty)
    },
    
    #' @description
    #' Enables padding for the tokenizer
    #' @param direction (str, optional, defaults to right) — The direction in which
    #'  to pad. Can be either `'right'` or `'left'`
    #' @param pad_to_multiple_of  (int, optional) — If specified, the padding length should 
    #'  always snap to the next multiple of the given value. For example if we were 
    #'  going to pad with a length of 250 but `pad_to_multiple_of=8` then we will 
    #'  pad to 256.
    #' @param pad_id (int, defaults to 0) — The id to be used when padding
    #' @param pad_type_id (int, defaults to 0) — The type id to be used when padding
    #' @param pad_token (str, defaults to `'[PAD]'`) — The pad token to be used when padding
    #' @param length (int, optional) — If specified, the length at which to pad. If not 
    #'  specified we pad using the size of the longest sequence in a batch.
    enable_padding = function(direction = "right", pad_id = 0L, pad_type_id = 0L, 
                             pad_token = "[PAD]", length = NULL, pad_to_multiple_of = NULL) {
      inputs <- list(
        direction = direction,
        pad_id = as.integer(pad_id),
        pad_token = pad_token,
        length = as.integer(length),
        pad_to_multiple_of = pad_to_multiple_of
      )
      inputs <- Filter(Negate(is.null), inputs)
      self$.tokenizer$enable_padding(inputs)
    },
    
    #' @description
    #' Disables padding
    no_padding = function() {
      self$.tokenizer$no_padding()
    },
    
    #' @description
    #' Enables truncation on the tokenizer
    #' @param max_length The maximum length at which to truncate.
    #' @param stride The length of the previous first sequence to be included
    #'        in the overflowing sequence. Default: `0`.
    #' @param strategy The strategy used for truncation. Can be one of:
    #'        "longest_first", "only_first", or "only_second". Default: "longest_first".
    #' @param direction The truncation direction. Default: "right".
    enable_truncation = function(max_length, stride = 0, strategy = "longest_first",
                                 direction = "right") {
      self$.tokenizer$enable_truncation(list(
        max_length = as.integer(max_length),
        stride = as.integer(stride),
        strategy = strategy,
        direction = direction
      ))
    },
    
    #' @description
    #' Disables truncation
    no_truncation = function() {
     self$.tokenizer$no_truncation() 
    },
    #' @description
    #' Gets the vocabulary size
    #' @param with_added_tokens Wether to count added tokens
    get_vocab_size = function(with_added_tokens = TRUE) {
     self$.tokenizer$get_vocab_size(with_added_tokens) 
    }
  ),
  active = list(
    #' @field pre_tokenizer instance of the pre-tokenizer
    pre_tokenizer = function(x) {
      if (missing(x)) {
        return(self$.tokenizer$get_pre_tokenizer())
      } 
      
      self$.tokenizer$set_pre_tokenizer(x$.pre_tokenizer)
      invisible(self$pre_tokenizer)
    },
    #' @field normalizer Gets the normalizer instance
    normalizer = function(x) {
      if (missing(x)) {
        return(self$.tokenizer$get_normalizer())
      } 
      
      self$.tokenizer$set_normalizer(x$.normalizer)
      invisible(self$normalizer)
    },
    #' @field post_processor Gets the post processor used by tokenizer
    post_processor = function(x) {
      if (missing(x)) {
        return(self$.tokenizer$get_post_processor())
      } 
      
      self$.tokenizer$set_post_processor(x$.processor)
      invisible(self$post_processor)
    },
    #' @field decoder Gets and sets the decoder
    decoder = function(x) {
      if (missing(x)) {
        return(self$.tokenizer$get_decoder())
      }
      
      self$.tokenizer$set_decoder(x$.decoder)
      invisible(self$decoder)
    },
    #' @field padding Gets padding configuration
    padding = function(x) {
      if (!missing(x)) {
        cli::cli_abort(gettext("Can't be set this way, use {.fn enable_padding}."))
      }
      
      self$.tokenizer$get_padding()
    },
    #' @field truncation Gets truncation configuration
    truncation = function(x) {
      if (!missing(x)) {
        cli::cli_abort(gettext("Can't be set this way, use {.fn enable_truncation}."))
      }
      
      self$.tokenizer$get_truncation()
    }
  )
)

tokenizer$from_file <- function(path) {
  path <- path.expand(path)
  tokenizer$new(RTokenizer$from_file(path))
}

tokenizer$from_pretrained <- function(identifier, revision = "main", auth_token = NULL) {
  if (!is.null(auth_token))
    withr::local_envvar(c(HUGGINGFACE_HUB_TOKEN = auth_token))
  
  rlang::check_installed("hfhub")
  path <- hfhub::hub_download(identifier, revision = revision, "tokenizer.json")
  
  tokenizer$new(RTokenizer$from_file(path))
}

test_that("Can use a post processor", {
  
  tok <- tokenizer$new(model_bpe$new())
  tok$pre_tokenizer <- pre_tokenizer_byte_level$new()
  
  text <- "model <- hello + world"
  tok$train_from_memory(text, trainer_bpe$new())
  
  expect_equal(
    tok$decode(tok$encode(text)$ids),
    "Ġmodel Ġ<- Ġhello Ġ+ Ġworld"
  )
  
  tok <- tokenizer$new(model_bpe$new())
  tok$pre_tokenizer <- pre_tokenizer_byte_level$new()
  
  text <- "model <- hello + world"
  tok$train_from_memory(text, trainer_bpe$new())
  
  tok$post_processor <- processor_byte_level$new(trim_offsets = FALSE)
  
  expect_equal(tok$decode(tok$encode(text)$ids), "Ġmodel Ġ<- Ġhello Ġ+ Ġworld")
  
})

test_that("Encoding", {
  expect_snapshot_error(encoding$new(1))
})

test_that("can get attributes from encoding", {
  
  tok <- tokenizer$from_file(test_path("assets/tokenizer.json"))
  encoding <- tok$encode("Hello world")
  
  expect_equal(encoding$ids, c(15496, 995))
  expect_equal(encoding$attention_mask, c(1,1))
})

test_that("train different tokenizer models", {
  tmp <- tempfile()
  writeLines(c("hello world", "bye bye"), tmp)
  
  models <- list(
    list(model = model_wordpiece$new(), trainer = trainer_wordpiece$new()),
    list(model = model_unigram$new(), trainer = trainer_unigram$new()),
    list(model = model_bpe$new(), trainer = trainer_bpe$new())
  )
  
  for (model in models) {
    tok <- tokenizer$new(model$model)
    tok$pre_tokenizer <- pre_tokenizer_whitespace$new()
    tok$train(tmp, model$trainer)  
    
    expect_true(is.integer(tok$encode("hello")$ids))
    expect_true(all(table(tok$encode("bye bye")$ids) == 2))
  }
  
})

test_that("R-level cli_abort messages are correctly translated in FR", {
  withr::with_envvar(c(HUGGINGFACE_HUB_CACHE = tempdir()), {
    try({
      tok <- tokenizer$from_pretrained("gpt2")
      temp_json <- tempfile(fileext = ".json")
      withr::with_language(lang = "fr",
                           expect_error(
                             tok$train(temp_json, temp_json),
                             regexp = "doit hériter de",
                             fixed = TRUE
                           ))
    })
  })
  
})

test_that("Can use a tokenizer from a file", {
  
  tok <- tokenizer$from_file(test_path("assets/tokenizer.json"))
  input <- "hello world"
  enc <- tok$encode(input)
  
  expect_equal(enc$ids, c(31373, 995))
  expect_equal(class(enc$ids), "integer")
  
  expect_equal(tok$decode(enc$ids), input)
})

test_that("batch encoder/decoder", {
  
  tok <- tokenizer$from_file(test_path("assets/tokenizer.json"))
  input <- c("hello world", "world hello")
  
  enc <- tok$encode_batch(input)
  
  # returns a list of encoding objects
  expect_equal(class(enc), "list")
  expect_equal(class(enc[[1]]), c("tok_encoding", "R6"))
  
  sequences <- lapply(enc, function(x) x$ids)
  
  expect_equal(tok$decode_batch(sequences), input)
})

test_that("from_pretrained", {
  skip_on_cran()
  tok <- tokenizer$from_pretrained("gpt2")
  input <- "hello world"
  enc <- tok$encode(input)
  
  expect_equal(tok$decode(enc$ids), input)
})

test_that("train a tokenizer on files", {
  
  tmp <- tempfile()
  writeLines(c("hello world", "bye bye"), tmp)
  
  tok <- tokenizer$new(model_bpe$new())
  tok$pre_tokenizer <- pre_tokenizer_whitespace$new()
  tok$train(tmp, trainer_bpe$new())
  
  expect_equal(tok$encode("hello")$ids, 17)
  expect_equal(tok$encode("world")$ids, 18)
  expect_equal(tok$encode("bye bye")$ids, c(10, 10))
})

test_that("can train a tokenizer from memory", {
  
  tok <- tokenizer$new(model_bpe$new())
  tok$pre_tokenizer <- pre_tokenizer_whitespace$new()
  tok$train_from_memory(c("hello world", "bye bye"), trainer_bpe$new())
  expect_equal(tok$get_vocab_size(), 19)
  
  expect_equal(tok$encode("hello")$ids, 17)
  expect_equal(tok$encode("world")$ids, 18)
  expect_equal(tok$encode("bye bye")$ids, c(10, 10))
  
  tok <- tokenizer$new(model_bpe$new())
  text <- "model <- hello + world"
  tok$train_from_memory(text, trainer_bpe$new())
  expect_equal(
    tok$decode(tok$encode(text)$ids),
    text
  )
})



test_that("can serialize a tokenizer and load back", {
  
  tok <- tokenizer$from_file(test_path("assets/tokenizer.json"))
  input <- "hello world"
  enc <- tok$encode(input)
  
  tmp <- tempfile(fileext = ".json")
  tok$save(tmp)
  
  tok2 <- tokenizer$from_file(tmp)
  enc2 <- tok$encode(input)
  
  expect_equal(enc$ids, enc2$ids)
})

test_that("enable padding works", {
  tok <- tokenizer$from_file(test_path("assets/tokenizer.json"))
  
  expect_null(tok$padding)
  
  tok$enable_padding(length = 20, pad_id = 5)
  input <- "hello world"
  enc <- tok$encode(input)
  
  expect_equal(length(enc$ids), 20)
  expect_equal(enc$ids[3], 5)
  
  expect_equal(tok$padding$length, 20)
  
  tok$no_padding()
  expect_null(tok$padding)
})

test_that("truncation works", {
  
  tok <- tokenizer$from_file(test_path("assets/tokenizer.json"))
  expect_null(tok$truncation)
  
  tok$enable_truncation(3)
  
  input <- "hello world I'm a new tokenizer called tok"
  enc <- tok$encode(input)
  
  expect_equal(length(enc$ids), 3)
  
  tok$no_truncation()
  expect_null(tok$padding)
  
  enc <- tok$encode(input)
  expect_true(length(enc$ids) > 3)
  
})

#' Encoding
#' @description 
#' Represents the output of a [tokenizer].
#' 
#' @examples
#' withr::with_envvar(c(HUGGINGFACE_HUB_CACHE = tempdir()), {
#' try({
#' tok <- tokenizer$from_pretrained("gpt2")
#' encoding <- tok$encode("Hello world")
#' encoding
#' })
#' })
#' @returns
#' An encoding object containing encoding information such as attention masks
#' and token ids.
#' 
#' @export
encoding <- R6::R6Class(
  classname = "tok_encoding",
  public = list(
    #' @field .encoding The underlying implementation pointer.
    .encoding = NULL,
    #' @description Initializes an encoding object (Not to use directly)
    #' @param encoding an encoding implementation object
    initialize = function(encoding) {
      if (inherits(encoding, "REncoding")) {
        self$.encoding <- encoding
      } else {
        cli::cli_abort(gettext("Expected class {.cls REncoding} but got {.cls {class(encoding)}}."))
      }
    }  
  ),
  active = list(
    #' @field ids The IDs are the main input to a Language Model. They are the 
    #'  token indices, the numerical representations that a LM understands.
    ids = function(x) {
      if (missing(x)) {
        self$.encoding$get_ids()
      }
    },
    #' @field attention_mask The attention mask used as input for transformers models.
    attention_mask = function(x) {
      if (missing(x)) {
        self$.encoding$get_attention_mask()
      }
    }
  )
)

#' Generic class for processors
#' @export
#' @family processors
tok_processor <- R6::R6Class(
  "tok_processor",
  public = list(
    #' @field .processor Internal pointer to processor object
    .processor = NULL,
    #' @description
    #' Initializes a tokenizer
    #' @param processor a raw pointer to a processor
    initialize = function(processor) {
      if (inherits(processor, "RPostProcessor")) {
        self$.processor <- processor
      } else {
        self$.processor <- RPostProcessor$new(processor)  
      }
    }
  )
)

#' Byte Level post processor
#' 
#' This post-processor takes care of trimming the offsets.
#' By default, the ByteLevel BPE might include whitespaces in the produced tokens. 
#' If you don’t want the offsets to include these whitespaces, then this 
#' PostProcessor must be used.
#'
#' @family processors
#' @export
processor_byte_level <- R6::R6Class(
  "tok_processor_byte_level",
  inherit = tok_processor,
  public = list(
    #' @description
    #' Initializes the byte level post processor
    #' @param trim_offsets Whether to trim the whitespaces from the produced offsets.
    initialize = function(trim_offsets = TRUE) {
      super$initialize(
        RPostProcessorByteLevel$new(trim_offsets)
      )
    }
  )
)

# Generated by extendr: Do not edit by hand

# nolint start

#
# This file was created with the following call:
#   .Call("wrap__make_tok_wrappers", use_symbols = TRUE, package_name = "tok")

#' @usage NULL
#' @useDynLib tok, .registration = TRUE
NULL

RModel <- new.env(parent = emptyenv())

RModel$new <- function(model) .Call(wrap__RModel__new, model)

#' @export
`$.RModel` <- function (self, name) { func <- RModel[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RModel` <- `$.RModel`

RModelBPE <- new.env(parent = emptyenv())

RModelBPE$new <- function(vocab, merges, cache_capacity, dropout, unk_token, continuing_subword_prefix, end_of_word_suffix, fuse_unk, byte_fallback) .Call(wrap__RModelBPE__new, vocab, merges, cache_capacity, dropout, unk_token, continuing_subword_prefix, end_of_word_suffix, fuse_unk, byte_fallback)

#' @export
`$.RModelBPE` <- function (self, name) { func <- RModelBPE[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RModelBPE` <- `$.RModelBPE`

RModelWordPiece <- new.env(parent = emptyenv())

RModelWordPiece$new <- function(vocab, unk_token, max_input_chars_per_word) .Call(wrap__RModelWordPiece__new, vocab, unk_token, max_input_chars_per_word)

#' @export
`$.RModelWordPiece` <- function (self, name) { func <- RModelWordPiece[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RModelWordPiece` <- `$.RModelWordPiece`

RModelUnigram <- new.env(parent = emptyenv())

RModelUnigram$new <- function(vocab, unk_id, byte_fallback) .Call(wrap__RModelUnigram__new, vocab, unk_id, byte_fallback)

#' @export
`$.RModelUnigram` <- function (self, name) { func <- RModelUnigram[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RModelUnigram` <- `$.RModelUnigram`

REncoding <- new.env(parent = emptyenv())

REncoding$len <- function() .Call(wrap__REncoding__len, self)

REncoding$get_ids <- function() .Call(wrap__REncoding__get_ids, self)

REncoding$get_tokens <- function() .Call(wrap__REncoding__get_tokens, self)

REncoding$get_type_ids <- function() .Call(wrap__REncoding__get_type_ids, self)

REncoding$get_attention_mask <- function() .Call(wrap__REncoding__get_attention_mask, self)

REncoding$get_special_tokens_mask <- function() .Call(wrap__REncoding__get_special_tokens_mask, self)

REncoding$get_word_ids <- function() .Call(wrap__REncoding__get_word_ids, self)

#' @export
`$.REncoding` <- function (self, name) { func <- REncoding[[name]]; environment(func) <- environment(); func }

#' @export
`[[.REncoding` <- `$.REncoding`

RTokenizer <- new.env(parent = emptyenv())

RTokenizer$new <- function(tokenizer) .Call(wrap__RTokenizer__new, tokenizer)

RTokenizer$from_file <- function(path) .Call(wrap__RTokenizer__from_file, path)

RTokenizer$from_model <- function(model) .Call(wrap__RTokenizer__from_model, model)

RTokenizer$encode <- function(sequence, pair, is_pretokenized, add_special_tokens) .Call(wrap__RTokenizer__encode, self, sequence, pair, is_pretokenized, add_special_tokens)

RTokenizer$decode <- function(ids, skip_special_tokens) .Call(wrap__RTokenizer__decode, self, ids, skip_special_tokens)

RTokenizer$encode_batch <- function(input, is_pretokenized, add_special_tokens) .Call(wrap__RTokenizer__encode_batch, self, input, is_pretokenized, add_special_tokens)

RTokenizer$decode_batch <- function(ids, skip_special_tokens) .Call(wrap__RTokenizer__decode_batch, self, ids, skip_special_tokens)

RTokenizer$set_pre_tokenizer <- function(pre_tokenizer) invisible(.Call(wrap__RTokenizer__set_pre_tokenizer, self, pre_tokenizer))

RTokenizer$get_pre_tokenizer <- function() .Call(wrap__RTokenizer__get_pre_tokenizer, self)

RTokenizer$set_post_processor <- function(post_processors) invisible(.Call(wrap__RTokenizer__set_post_processor, self, post_processors))

RTokenizer$get_post_processor <- function() .Call(wrap__RTokenizer__get_post_processor, self)

RTokenizer$set_normalizer <- function(normalizer) invisible(.Call(wrap__RTokenizer__set_normalizer, self, normalizer))

RTokenizer$get_normalizer <- function() .Call(wrap__RTokenizer__get_normalizer, self)

RTokenizer$set_decoder <- function(decoder) invisible(.Call(wrap__RTokenizer__set_decoder, self, decoder))

RTokenizer$get_decoder <- function() .Call(wrap__RTokenizer__get_decoder, self)

RTokenizer$train_from_files <- function(trainer, files) invisible(.Call(wrap__RTokenizer__train_from_files, self, trainer, files))

RTokenizer$train_from_sequences <- function(trainer, sequences) invisible(.Call(wrap__RTokenizer__train_from_sequences, self, trainer, sequences))

RTokenizer$save <- function(path, pretty) invisible(.Call(wrap__RTokenizer__save, self, path, pretty))

RTokenizer$enable_padding <- function(padding) invisible(.Call(wrap__RTokenizer__enable_padding, self, padding))

RTokenizer$get_padding <- function() .Call(wrap__RTokenizer__get_padding, self)

RTokenizer$no_padding <- function() invisible(.Call(wrap__RTokenizer__no_padding, self))

RTokenizer$enable_truncation <- function(truncation) invisible(.Call(wrap__RTokenizer__enable_truncation, self, truncation))

RTokenizer$get_truncation <- function() .Call(wrap__RTokenizer__get_truncation, self)

RTokenizer$no_truncation <- function() invisible(.Call(wrap__RTokenizer__no_truncation, self))

RTokenizer$get_vocab_size <- function(with_added_tokens) .Call(wrap__RTokenizer__get_vocab_size, self, with_added_tokens)

#' @export
`$.RTokenizer` <- function (self, name) { func <- RTokenizer[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RTokenizer` <- `$.RTokenizer`

RTrainer <- new.env(parent = emptyenv())

RTrainer$new <- function(trainer) .Call(wrap__RTrainer__new, trainer)

#' @export
`$.RTrainer` <- function (self, name) { func <- RTrainer[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RTrainer` <- `$.RTrainer`

RTrainerBPE <- new.env(parent = emptyenv())

RTrainerBPE$new <- function(vocab_size, min_frequency, show_progress, special_tokens, limit_alphabet, initial_alphabet, continuing_subword_prefix, end_of_word_suffix, max_token_length) .Call(wrap__RTrainerBPE__new, vocab_size, min_frequency, show_progress, special_tokens, limit_alphabet, initial_alphabet, continuing_subword_prefix, end_of_word_suffix, max_token_length)

#' @export
`$.RTrainerBPE` <- function (self, name) { func <- RTrainerBPE[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RTrainerBPE` <- `$.RTrainerBPE`

RTrainerWordPiece <- new.env(parent = emptyenv())

RTrainerWordPiece$new <- function(vocab_size, min_frequency, show_progress, special_tokens, limit_alphabet, initial_alphabet, continuing_subword_prefix, end_of_word_suffix) .Call(wrap__RTrainerWordPiece__new, vocab_size, min_frequency, show_progress, special_tokens, limit_alphabet, initial_alphabet, continuing_subword_prefix, end_of_word_suffix)

#' @export
`$.RTrainerWordPiece` <- function (self, name) { func <- RTrainerWordPiece[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RTrainerWordPiece` <- `$.RTrainerWordPiece`

RTrainerUnigram <- new.env(parent = emptyenv())

RTrainerUnigram$new <- function(vocab_size, show_progress, special_tokens, initial_alphabet, shrinking_factor, unk_token, max_piece_length, n_sub_iterations) .Call(wrap__RTrainerUnigram__new, vocab_size, show_progress, special_tokens, initial_alphabet, shrinking_factor, unk_token, max_piece_length, n_sub_iterations)

#' @export
`$.RTrainerUnigram` <- function (self, name) { func <- RTrainerUnigram[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RTrainerUnigram` <- `$.RTrainerUnigram`

RPreTokenizer <- new.env(parent = emptyenv())

RPreTokenizer$new <- function(pre_tokenizer) .Call(wrap__RPreTokenizer__new, pre_tokenizer)

#' @export
`$.RPreTokenizer` <- function (self, name) { func <- RPreTokenizer[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RPreTokenizer` <- `$.RPreTokenizer`

RPreTokenizerWhitespace <- new.env(parent = emptyenv())

RPreTokenizerWhitespace$new <- function() .Call(wrap__RPreTokenizerWhitespace__new)

#' @export
`$.RPreTokenizerWhitespace` <- function (self, name) { func <- RPreTokenizerWhitespace[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RPreTokenizerWhitespace` <- `$.RPreTokenizerWhitespace`

RPreTokenizerByteLevel <- new.env(parent = emptyenv())

RPreTokenizerByteLevel$new <- function(add_prefix_space, use_regex) .Call(wrap__RPreTokenizerByteLevel__new, add_prefix_space, use_regex)

#' @export
`$.RPreTokenizerByteLevel` <- function (self, name) { func <- RPreTokenizerByteLevel[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RPreTokenizerByteLevel` <- `$.RPreTokenizerByteLevel`

RNormalizer <- new.env(parent = emptyenv())

RNormalizer$new <- function(normalizer) .Call(wrap__RNormalizer__new, normalizer)

#' @export
`$.RNormalizer` <- function (self, name) { func <- RNormalizer[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RNormalizer` <- `$.RNormalizer`

RNormalizerNFC <- new.env(parent = emptyenv())

RNormalizerNFC$new <- function() .Call(wrap__RNormalizerNFC__new)

#' @export
`$.RNormalizerNFC` <- function (self, name) { func <- RNormalizerNFC[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RNormalizerNFC` <- `$.RNormalizerNFC`

RNormalizerNFKC <- new.env(parent = emptyenv())

RNormalizerNFKC$new <- function() .Call(wrap__RNormalizerNFKC__new)

#' @export
`$.RNormalizerNFKC` <- function (self, name) { func <- RNormalizerNFKC[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RNormalizerNFKC` <- `$.RNormalizerNFKC`

RPostProcessor <- new.env(parent = emptyenv())

RPostProcessor$new <- function(post_processor) .Call(wrap__RPostProcessor__new, post_processor)

#' @export
`$.RPostProcessor` <- function (self, name) { func <- RPostProcessor[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RPostProcessor` <- `$.RPostProcessor`

RPostProcessorByteLevel <- new.env(parent = emptyenv())

RPostProcessorByteLevel$new <- function(trim_offsets) .Call(wrap__RPostProcessorByteLevel__new, trim_offsets)

#' @export
`$.RPostProcessorByteLevel` <- function (self, name) { func <- RPostProcessorByteLevel[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RPostProcessorByteLevel` <- `$.RPostProcessorByteLevel`

RDecoder <- new.env(parent = emptyenv())

RDecoder$new <- function(decoder) .Call(wrap__RDecoder__new, decoder)

#' @export
`$.RDecoder` <- function (self, name) { func <- RDecoder[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RDecoder` <- `$.RDecoder`

RDecoderByteLevel <- new.env(parent = emptyenv())

RDecoderByteLevel$new <- function() .Call(wrap__RDecoderByteLevel__new)

#' @export
`$.RDecoderByteLevel` <- function (self, name) { func <- RDecoderByteLevel[[name]]; environment(func) <- environment(); func }

#' @export
`[[.RDecoderByteLevel` <- `$.RDecoderByteLevel`


# nolint end

#' Generic class for decoders
#' @export
#' @family decoders
tok_decoder <- R6::R6Class(
  "tok_decoder",
  public = list(
    #' @field .decoder The raw pointer to the decoder
    .decoder = NULL,
    #' @description
    #' Initializes a decoder
    #' @param decoder a raw decoder pointer
    initialize = function(decoder) {
      if (inherits(decoder, "RDecoder")) {
        self$.decoder <- decoder
      } else {
        self$.decoder <- RDecoder$new(decoder)
      }
    }
  )
)

#' Byte level decoder
#' 
#' This decoder is to be used with the [pre_tokenizer_byte_level].
#' @export
#' @family decoders
decoder_byte_level <- R6::R6Class(
  "tok_decoder_byte_level",
  inherit = tok_decoder,
  public = list(
    #' @description
    #' Initializes a byte level decoder
    initialize = function() {
      super$initialize(RDecoderByteLevel$new())
    }
  )
)

#' Generic class for normalizers
#' @export
#' @family normalizers
#' @export
tok_normalizer <- R6::R6Class(
  "tok_normalizer",
  public = list(
    #' @field .normalizer Internal pointer to normalizer object
    .normalizer = NULL,
    #' @description
    #' Initializes a tokenizer
    #' @param normalizer a raw pointer to a tokenizer
    initialize = function(normalizer) {
      if (inherits(normalizer, "RNormalizer")) {
        self$.normalizer <- normalizer
      } else {
        self$.normalizer <- RNormalizer$new(normalizer)  
      }
    }
  )
)

#' NFC normalizer
#' @export
#' @family normalizers
normalizer_nfc <- R6::R6Class(
  "tok_normalizer_nfc",
  inherit = tok_normalizer,
  public = list(
    #' @description
    #' Initializes the NFC normalizer
    initialize = function() {
      super$initialize(RNormalizerNFC$new())
    }  
  )
)

#' NFKC normalizer
#' @export
#' @family normalizers
normalizer_nfkc <- R6::R6Class(
  "tok_normalizer_nfkc",
  inherit = tok_normalizer,
  public = list(
    #' @description
    #' Initializes the NFKC normalizer
    initialize = function() {
      super$initialize(RNormalizerNFKC$new())
    }  
  )
)

#' Generic class for tokenizers
#' @export
#' @family pre_tokenizer
pre_tokenizer <- R6::R6Class(
  "tok_pre_tokenizer",
  public = list(
    #' @field .pre_tokenizer Internal pointer to tokenizer object
    .pre_tokenizer = NULL,
    #' @description
    #' Initializes a tokenizer
    #' @param pre_tokenizer a raw pointer to a tokenizer
    initialize = function(pre_tokenizer) {
      if (inherits(pre_tokenizer, "RPreTokenizer")) {
        self$.pre_tokenizer <- pre_tokenizer
      } else {
        self$.pre_tokenizer <- RPreTokenizer$new(pre_tokenizer)  
      }
    }
  )
)

#' This pre-tokenizer simply splits using the following regex: `\w+|[^\w\s]+`
#' @export
#' @family pre_tokenizer
pre_tokenizer_whitespace <- R6::R6Class(
  "tok_pre_tokenizer_whitespace",
  inherit = pre_tokenizer,
  public = list(
    #' @description
    #' Initializes the whistespace tokenizer
    initialize = function() {
      super$initialize(RPreTokenizerWhitespace$new())
    }
  )
)

#' Byte level pre tokenizer
#' 
#' This pre-tokenizer takes care of replacing all bytes of the given string with 
#' a corresponding representation, as well as splitting into words.
#' @export
#' @family pre_tokenizer
pre_tokenizer_byte_level <- R6::R6Class(
  "tok_pre_tokenizer_byte_level",
  inherit = pre_tokenizer,
  public = list(
    #' @description
    #' Initializes the bytelevel tokenizer
    #' @param add_prefix_space Whether to add a space to the first word
    #' @param use_regex Set this to False to prevent this pre_tokenizer from using 
    #'   the GPT2 specific regexp for spliting on whitespace.
    initialize = function(add_prefix_space = TRUE, use_regex = TRUE) {
      super$initialize(RPreTokenizerByteLevel$new(
        add_prefix_space, 
        use_regex
      ))
    }
  )
)

#' Generic training class
#' @export
#' @family trainer
tok_trainer <- R6::R6Class(
  "tok_trainer",
  public = list(
    #' @field .trainer a pointer to a raw trainer
    .trainer = NULL,
    #' @description
    #' Initializes a generic trainer from a raw trainer
    #' @param trainer raw trainer (internal)
    initialize = function(trainer) {
      self$.trainer <- RTrainer$new(trainer)
    }  
  )
)

#' BPE trainer
#' @export
#' @family trainer
trainer_bpe <- R6::R6Class(
  "tok_trainer_bpe",
  inherit = tok_trainer,
  public = list(
    #' @description
    #' Constrcutor for the BPE trainer
    #' @param vocab_size The size of the final vocabulary, including all tokens and alphabet.
    #'        Default: `NULL`.
    #' @param min_frequency The minimum frequency a pair should have in order to be merged.
    #'        Default: `NULL`.
    #' @param show_progress Whether to show progress bars while training. Default: `TRUE`.
    #' @param special_tokens A list of special tokens the model should be aware of.
    #'        Default: `NULL`.
    #' @param limit_alphabet The maximum number of different characters to keep in the alphabet.
    #'        Default: `NULL`.
    #' @param initial_alphabet A list of characters to include in the initial alphabet,
    #'        even if not seen in the training dataset. Default: `NULL`.
    #' @param continuing_subword_prefix A prefix to be used for every subword that is not a beginning-of-word.
    #'        Default: `NULL`.
    #' @param end_of_word_suffix A suffix to be used for every subword that is an end-of-word.
    #'        Default: `NULL`.
    #' @param max_token_length Prevents creating tokens longer than the specified size.
    #'        Default: `NULL`.
    initialize = function(vocab_size = NULL, 
                          min_frequency = NULL, 
                          show_progress = NULL, 
                          special_tokens = NULL, 
                          limit_alphabet = NULL, 
                          initial_alphabet = NULL,
                          continuing_subword_prefix = NULL, 
                          end_of_word_suffix = NULL,
                          max_token_length = NULL) {
      super$initialize(RTrainerBPE$new(
        vocab_size = vocab_size,
        min_frequency = min_frequency,
        show_progress = show_progress,
        special_tokens = special_tokens,
        limit_alphabet = limit_alphabet,
        initial_alphabet = initial_alphabet,
        continuing_subword_prefix = continuing_subword_prefix,
        end_of_word_suffix = end_of_word_suffix,
        max_token_length = max_token_length
      ))
    }
  )
)

#' WordPiece tokenizer trainer
#' @export
#' @family trainer
trainer_wordpiece <- R6::R6Class(
  "tok_trainer_wordpiece",
  inherit = tok_trainer,
  public = list(
    #' @description
    #' Constructor for the WordPiece tokenizer trainer
    #' 
    #' @param vocab_size The size of the final vocabulary, including all tokens and alphabet.
    #'        Default: `NULL`.
    #' @param min_frequency The minimum frequency a pair should have in order to be merged.
    #'        Default: `NULL`.
    #' @param show_progress Whether to show progress bars while training. Default: `TRUE`.
    #' @param special_tokens A list of special tokens the model should be aware of.
    #'        Default: `NULL`.
    #' @param limit_alphabet The maximum number of different characters to keep in the alphabet.
    #'        Default: `NULL`.
    #' @param initial_alphabet A list of characters to include in the initial alphabet,
    #'        even if not seen in the training dataset. If the strings contain more than
    #'        one character, only the first one is kept. Default: `NULL`.
    #' @param continuing_subword_prefix A prefix to be used for every subword that is not a beginning-of-word.
    #'        Default: `NULL`.
    #' @param end_of_word_suffix A suffix to be used for every subword that is an end-of-word.
    #'        Default: `NULL`.
    initialize = function(
    vocab_size = 30000,
    min_frequency = 0,
    show_progress = FALSE,
    special_tokens = NULL,
    limit_alphabet = NULL,
    initial_alphabet = NULL,
    continuing_subword_prefix = "##",
    end_of_word_suffix = NULL
    ) {
      super$initialize(RTrainerWordPiece$new(
        vocab_size = vocab_size,
        min_frequency = min_frequency,
        show_progress = show_progress,
        special_tokens = special_tokens,
        limit_alphabet = limit_alphabet,
        initial_alphabet = initial_alphabet,
        continuing_subword_prefix = continuing_subword_prefix,
        end_of_word_suffix = end_of_word_suffix
      ))
    }
  )
)

#' Unigram tokenizer trainer
#' @export
#' @family trainer
trainer_unigram <- R6::R6Class(
  "tok_trainer_unigram",
  inherit = tok_trainer,
  public = list(
    #' @description
    #' Constructor for the Unigram tokenizer
    #' @param vocab_size The size of the final vocabulary, including all tokens and alphabet.
    #' @param show_progress Whether to show progress bars while training.
    #' @param special_tokens A list of special tokens the model should be aware of.
    #' @param initial_alphabet A list of characters to include in the initial alphabet,
    #'        even if not seen in the training dataset. If the strings contain more than
    #'        one character, only the first one is kept.
    #' @param shrinking_factor The shrinking factor used at each step of training
    #'        to prune the vocabulary.
    #' @param unk_token The token used for out-of-vocabulary tokens.
    #' @param max_piece_length The maximum length of a given token.
    #' @param n_sub_iterations The number of iterations of the EM algorithm to perform
    #'        before pruning the vocabulary.
    initialize = function(
    vocab_size = 8000,
    show_progress = TRUE,
    special_tokens = NULL,
    shrinking_factor = 0.75,
    unk_token = NULL,
    max_piece_length = 16,
    n_sub_iterations = 2
    ) {
      super$initialize(RTrainerUnigram$new(
        vocab_size = vocab_size,
        show_progress = show_progress,
        special_tokens = special_tokens,
        shrinking_factor = shrinking_factor,
        unk_token = unk_token,
        max_piece_length = max_piece_length,
        n_sub_iterations = n_sub_iterations,
        initial_alphabet = NULL
      ))
    }
  )
)

test_that("Can initialize a tokenizer different models", {
  models <- list(
    model_bpe$new(),
    model_wordpiece$new(),
    model_unigram$new(),
    model_unigram$new(vocab = c("hello" = 10, "bye" = 12), 1L),
    model_unigram$new(vocab = list("hello" = 10, "bye" = 12), 1L)
  )
  
  for (model in models) {
    tok <- tokenizer$new(model)
    expect_true(inherits(tok, "tok_tokenizer"))
  }
})

test_that("can set a pre_tokenizer for a tokenizer", {
  
  tok <- tokenizer$new(model_bpe$new())
  tok$pre_tokenizer <- pre_tokenizer_whitespace$new()
  
  expect_true(inherits(tok$pre_tokenizer, "tok_pre_tokenizer"))
  
})

test_that("normalizers can be set", {
  tok <- tokenizer$new(model_bpe$new())
  tok$normalizer <- normalizer_nfc$new()
  expect_true(inherits(tok$normalizer, "tok_normalizer"))
  
  tok <- tokenizer$new(model_bpe$new())
  tok$normalizer <- normalizer_nfkc$new()
  expect_true(inherits(tok$normalizer, "tok_normalizer"))
})

#!/bin/sh -e

cd src/rust
cargo vendor

# wit-bindgen uses edition 2024, which Cargo < 1.85 cannot parse.
# Since it is never compiled for our targets (only needed for WASI),
# we patch it to edition 2021 so older Cargo can read the manifest.
sed -i.bak 's/^edition = "2024"/edition = "2021"/' vendor/wit-bindgen/Cargo.toml
rm -f vendor/wit-bindgen/Cargo.toml.bak

# c.f. https://reproducible-builds.org/docs/archives/
# --no-xattrs and COPYFILE_DISABLE prevent macOS extended attributes
# (e.g. com.apple.provenance) from being stored in the archive, which
# would cause warnings when extracted with GNU tar on Linux.
COPYFILE_DISABLE=1 tar \
  --xz \
  --create \
  --no-xattrs \
  --file=vendor.tar.xz \
  vendor

echo
echo
echo "#############################################"
echo "#                                           #"
echo "#  UPDATE src/cargo_vendor_config.toml !!!  #"
echo "#                                           #"
echo "#############################################"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tokenizer.R
\name{tokenizer}
\alias{tokenizer}
\title{Tokenizer}
\value{
A tokenizer that can be used for encoding character strings or decoding
integers.
}
\description{
A Tokenizer works as a pipeline. It processes some raw text as input and outputs
an \link{encoding}.
}
\examples{
withr::with_envvar(c(HUGGINGFACE_HUB_CACHE = tempdir()), {
try({
tok <- tokenizer$from_pretrained("gpt2")
tok$encode("Hello world")$ids
})
})

}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{.tokenizer}}{(unsafe usage) Lower level pointer to tokenizer}
}
\if{html}{\out{</div>}}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{pre_tokenizer}}{instance of the pre-tokenizer}

\item{\code{normalizer}}{Gets the normalizer instance}

\item{\code{post_processor}}{Gets the post processor used by tokenizer}

\item{\code{decoder}}{Gets and sets the decoder}

\item{\code{padding}}{Gets padding configuration}

\item{\code{truncation}}{Gets truncation configuration}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_tokenizer-new}{\code{tokenizer$new()}}
\item \href{#method-tok_tokenizer-encode}{\code{tokenizer$encode()}}
\item \href{#method-tok_tokenizer-decode}{\code{tokenizer$decode()}}
\item \href{#method-tok_tokenizer-encode_batch}{\code{tokenizer$encode_batch()}}
\item \href{#method-tok_tokenizer-decode_batch}{\code{tokenizer$decode_batch()}}
\item \href{#method-tok_tokenizer-from_file}{\code{tokenizer$from_file()}}
\item \href{#method-tok_tokenizer-from_pretrained}{\code{tokenizer$from_pretrained()}}
\item \href{#method-tok_tokenizer-train}{\code{tokenizer$train()}}
\item \href{#method-tok_tokenizer-train_from_memory}{\code{tokenizer$train_from_memory()}}
\item \href{#method-tok_tokenizer-save}{\code{tokenizer$save()}}
\item \href{#method-tok_tokenizer-enable_padding}{\code{tokenizer$enable_padding()}}
\item \href{#method-tok_tokenizer-no_padding}{\code{tokenizer$no_padding()}}
\item \href{#method-tok_tokenizer-enable_truncation}{\code{tokenizer$enable_truncation()}}
\item \href{#method-tok_tokenizer-no_truncation}{\code{tokenizer$no_truncation()}}
\item \href{#method-tok_tokenizer-get_vocab_size}{\code{tokenizer$get_vocab_size()}}
\item \href{#method-tok_tokenizer-clone}{\code{tokenizer$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-new}{}}}
\subsection{Method \code{new()}}{
Initializes a tokenizer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$new(tokenizer)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{tokenizer}}{Will be cloned to initialize a new tokenizer}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-encode"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-encode}{}}}
\subsection{Method \code{encode()}}{
Encode the given sequence and pair. This method can process raw text sequences
as well as already pre-tokenized sequences.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$encode(
  sequence,
  pair = NULL,
  is_pretokenized = FALSE,
  add_special_tokens = TRUE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{sequence}}{The main input sequence we want to encode. This sequence can
be either raw text or pre-tokenized, according to the is_pretokenized argument}

\item{\code{pair}}{An optional input sequence. The expected format is the same
that for sequence.}

\item{\code{is_pretokenized}}{Whether the input is already pre-tokenized}

\item{\code{add_special_tokens}}{Whether to add the special tokens}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-decode"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-decode}{}}}
\subsection{Method \code{decode()}}{
Decode the given list of ids back to a string
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$decode(ids, skip_special_tokens = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ids}}{The list of ids that we want to decode}

\item{\code{skip_special_tokens}}{Whether the special tokens should be removed from the decoded string}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-encode_batch"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-encode_batch}{}}}
\subsection{Method \code{encode_batch()}}{
Encodes a batch of sequences. Returns a list of \link{encoding}s.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$encode_batch(
  input,
  is_pretokenized = FALSE,
  add_special_tokens = TRUE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{input}}{A list of single sequences or pair sequences to encode. Each
sequence can be either raw text or pre-tokenized, according to the is_pretokenized
argument.}

\item{\code{is_pretokenized}}{Whether the input is already pre-tokenized}

\item{\code{add_special_tokens}}{Whether to add the special tokens}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-decode_batch"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-decode_batch}{}}}
\subsection{Method \code{decode_batch()}}{
Decode a batch of ids back to their corresponding string
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$decode_batch(sequences, skip_special_tokens = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{sequences}}{The batch of sequences we want to decode}

\item{\code{skip_special_tokens}}{Whether the special tokens should be removed from the decoded strings}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-from_file"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-from_file}{}}}
\subsection{Method \code{from_file()}}{
Creates a tokenizer from the path of a serialized tokenizer.
This is a static method and should be called instead of \verb{$new} when initializing
the tokenizer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$from_file(path)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{path}}{Path to tokenizer.json file}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-from_pretrained"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-from_pretrained}{}}}
\subsection{Method \code{from_pretrained()}}{
Instantiate a new Tokenizer from an existing file on the Hugging Face Hub.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$from_pretrained(identifier, revision = "main", auth_token = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{identifier}}{The identifier of a Model on the Hugging Face Hub, that
contains a tokenizer.json file}

\item{\code{revision}}{A branch or commit id}

\item{\code{auth_token}}{An optional auth token used to access private repositories
on the Hugging Face Hub}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-train"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-train}{}}}
\subsection{Method \code{train()}}{
Train the Tokenizer using the given files.
Reads the files line by line, while keeping all the whitespace, even new lines.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$train(files, trainer)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{files}}{character vector of file paths.}

\item{\code{trainer}}{an instance of a trainer object, specific to that tokenizer type.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-train_from_memory"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-train_from_memory}{}}}
\subsection{Method \code{train_from_memory()}}{
Train the tokenizer on a chracter vector of texts
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$train_from_memory(texts, trainer)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{texts}}{a character vector of texts.}

\item{\code{trainer}}{an instance of a trainer object, specific to that tokenizer type.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-save"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-save}{}}}
\subsection{Method \code{save()}}{
Saves the tokenizer to a json file
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$save(path, pretty = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{path}}{A path to a file in which to save the serialized tokenizer.}

\item{\code{pretty}}{Whether the JSON file should be pretty formatted.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-enable_padding"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-enable_padding}{}}}
\subsection{Method \code{enable_padding()}}{
Enables padding for the tokenizer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$enable_padding(
  direction = "right",
  pad_id = 0L,
  pad_type_id = 0L,
  pad_token = "[PAD]",
  length = NULL,
  pad_to_multiple_of = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{direction}}{(str, optional, defaults to right) — The direction in which
to pad. Can be either \code{'right'} or \code{'left'}}

\item{\code{pad_id}}{(int, defaults to 0) — The id to be used when padding}

\item{\code{pad_type_id}}{(int, defaults to 0) — The type id to be used when padding}

\item{\code{pad_token}}{(str, defaults to \code{'[PAD]'}) — The pad token to be used when padding}

\item{\code{length}}{(int, optional) — If specified, the length at which to pad. If not
specified we pad using the size of the longest sequence in a batch.}

\item{\code{pad_to_multiple_of}}{(int, optional) — If specified, the padding length should
always snap to the next multiple of the given value. For example if we were
going to pad with a length of 250 but \code{pad_to_multiple_of=8} then we will
pad to 256.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-no_padding"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-no_padding}{}}}
\subsection{Method \code{no_padding()}}{
Disables padding
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$no_padding()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-enable_truncation"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-enable_truncation}{}}}
\subsection{Method \code{enable_truncation()}}{
Enables truncation on the tokenizer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$enable_truncation(
  max_length,
  stride = 0,
  strategy = "longest_first",
  direction = "right"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{max_length}}{The maximum length at which to truncate.}

\item{\code{stride}}{The length of the previous first sequence to be included
in the overflowing sequence. Default: \code{0}.}

\item{\code{strategy}}{The strategy used for truncation. Can be one of:
"longest_first", "only_first", or "only_second". Default: "longest_first".}

\item{\code{direction}}{The truncation direction. Default: "right".}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-no_truncation"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-no_truncation}{}}}
\subsection{Method \code{no_truncation()}}{
Disables truncation
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$no_truncation()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-get_vocab_size"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-get_vocab_size}{}}}
\subsection{Method \code{get_vocab_size()}}{
Gets the vocabulary size
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$get_vocab_size(with_added_tokens = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{with_added_tokens}}{Wether to count added tokens}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_tokenizer-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_tokenizer-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tokenizer$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalizers.R
\name{normalizer_nfkc}
\alias{normalizer_nfkc}
\title{NFKC normalizer}
\description{
NFKC normalizer

NFKC normalizer
}
\seealso{
Other normalizers: 
\code{\link{normalizer_nfc}},
\code{\link{tok_normalizer}}
}
\concept{normalizers}
\section{Super class}{
\code{\link[tok:tok_normalizer]{tok::tok_normalizer}} -> \code{tok_normalizer_nfkc}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_normalizer_nfkc-new}{\code{normalizer_nfkc$new()}}
\item \href{#method-tok_normalizer_nfkc-clone}{\code{normalizer_nfkc$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_normalizer_nfkc-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_normalizer_nfkc-new}{}}}
\subsection{Method \code{new()}}{
Initializes the NFKC normalizer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{normalizer_nfkc$new()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_normalizer_nfkc-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_normalizer_nfkc-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{normalizer_nfkc$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models.R
\name{model_wordpiece}
\alias{model_wordpiece}
\title{An implementation of the WordPiece algorithm}
\description{
An implementation of the WordPiece algorithm

An implementation of the WordPiece algorithm
}
\seealso{
Other model: 
\code{\link{model_bpe}},
\code{\link{model_unigram}},
\code{\link{tok_model}}
}
\concept{model}
\section{Super class}{
\code{\link[tok:tok_model]{tok::tok_model}} -> \code{tok_model_wordpiece}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_model_wordpiece-new}{\code{model_wordpiece$new()}}
\item \href{#method-tok_model_wordpiece-clone}{\code{model_wordpiece$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_model_wordpiece-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_model_wordpiece-new}{}}}
\subsection{Method \code{new()}}{
Constructor for the wordpiece tokenizer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{model_wordpiece$new(
  vocab = NULL,
  unk_token = NULL,
  max_input_chars_per_word = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vocab}}{A dictionary of string keys and their corresponding ids.
Default: \code{NULL}.}

\item{\code{unk_token}}{The unknown token to be used by the model.
Default: \code{NULL}.}

\item{\code{max_input_chars_per_word}}{The maximum number of characters to allow in a single word.
Default: \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_model_wordpiece-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_model_wordpiece-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{model_wordpiece$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decoders.R
\name{tok_decoder}
\alias{tok_decoder}
\title{Generic class for decoders}
\description{
Generic class for decoders

Generic class for decoders
}
\seealso{
Other decoders: 
\code{\link{decoder_byte_level}}
}
\concept{decoders}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{.decoder}}{The raw pointer to the decoder}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_decoder-new}{\code{tok_decoder$new()}}
\item \href{#method-tok_decoder-clone}{\code{tok_decoder$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_decoder-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_decoder-new}{}}}
\subsection{Method \code{new()}}{
Initializes a decoder
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tok_decoder$new(decoder)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{decoder}}{a raw decoder pointer}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_decoder-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_decoder-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tok_decoder$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalizers.R
\name{tok_normalizer}
\alias{tok_normalizer}
\title{Generic class for normalizers}
\description{
Generic class for normalizers

Generic class for normalizers
}
\seealso{
Other normalizers: 
\code{\link{normalizer_nfc}},
\code{\link{normalizer_nfkc}}
}
\concept{normalizers}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{.normalizer}}{Internal pointer to normalizer object}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_normalizer-new}{\code{tok_normalizer$new()}}
\item \href{#method-tok_normalizer-clone}{\code{tok_normalizer$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_normalizer-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_normalizer-new}{}}}
\subsection{Method \code{new()}}{
Initializes a tokenizer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tok_normalizer$new(normalizer)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{normalizer}}{a raw pointer to a tokenizer}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_normalizer-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_normalizer-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tok_normalizer$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models.R
\name{model_unigram}
\alias{model_unigram}
\title{An implementation of the Unigram algorithm}
\description{
An implementation of the Unigram algorithm

An implementation of the Unigram algorithm
}
\seealso{
Other model: 
\code{\link{model_bpe}},
\code{\link{model_wordpiece}},
\code{\link{tok_model}}
}
\concept{model}
\section{Super class}{
\code{\link[tok:tok_model]{tok::tok_model}} -> \code{tok_model_unigram}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_model_unigram-new}{\code{model_unigram$new()}}
\item \href{#method-tok_model_unigram-clone}{\code{model_unigram$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_model_unigram-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_model_unigram-new}{}}}
\subsection{Method \code{new()}}{
Constructor for Unigram Model
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{model_unigram$new(vocab = NULL, unk_id = NULL, byte_fallback = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vocab}}{A dictionary of string keys and their corresponding relative score.
Default: \code{NULL}.}

\item{\code{unk_id}}{The unknown token id to be used by the model.
Default: \code{NULL}.}

\item{\code{byte_fallback}}{Whether to use byte-fallback trick. Default: \code{FALSE}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_model_unigram-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_model_unigram-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{model_unigram$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/post_processors.R
\name{processor_byte_level}
\alias{processor_byte_level}
\title{Byte Level post processor}
\description{
Byte Level post processor

Byte Level post processor
}
\details{
This post-processor takes care of trimming the offsets.
By default, the ByteLevel BPE might include whitespaces in the produced tokens.
If you don’t want the offsets to include these whitespaces, then this
PostProcessor must be used.
}
\seealso{
Other processors: 
\code{\link{tok_processor}}
}
\concept{processors}
\section{Super class}{
\code{\link[tok:tok_processor]{tok::tok_processor}} -> \code{tok_processor_byte_level}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_processor_byte_level-new}{\code{processor_byte_level$new()}}
\item \href{#method-tok_processor_byte_level-clone}{\code{processor_byte_level$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_processor_byte_level-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_processor_byte_level-new}{}}}
\subsection{Method \code{new()}}{
Initializes the byte level post processor
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{processor_byte_level$new(trim_offsets = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{trim_offsets}}{Whether to trim the whitespaces from the produced offsets.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_processor_byte_level-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_processor_byte_level-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{processor_byte_level$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pre_tokenizers.R
\name{pre_tokenizer_byte_level}
\alias{pre_tokenizer_byte_level}
\title{Byte level pre tokenizer}
\description{
Byte level pre tokenizer

Byte level pre tokenizer
}
\details{
This pre-tokenizer takes care of replacing all bytes of the given string with
a corresponding representation, as well as splitting into words.
}
\seealso{
Other pre_tokenizer: 
\code{\link{pre_tokenizer}},
\code{\link{pre_tokenizer_whitespace}}
}
\concept{pre_tokenizer}
\section{Super class}{
\code{tok::tok_pre_tokenizer} -> \code{tok_pre_tokenizer_byte_level}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_pre_tokenizer_byte_level-new}{\code{pre_tokenizer_byte_level$new()}}
\item \href{#method-tok_pre_tokenizer_byte_level-clone}{\code{pre_tokenizer_byte_level$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_pre_tokenizer_byte_level-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_pre_tokenizer_byte_level-new}{}}}
\subsection{Method \code{new()}}{
Initializes the bytelevel tokenizer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{pre_tokenizer_byte_level$new(add_prefix_space = TRUE, use_regex = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{add_prefix_space}}{Whether to add a space to the first word}

\item{\code{use_regex}}{Set this to False to prevent this pre_tokenizer from using
the GPT2 specific regexp for spliting on whitespace.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_pre_tokenizer_byte_level-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_pre_tokenizer_byte_level-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{pre_tokenizer_byte_level$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainers.R
\name{trainer_bpe}
\alias{trainer_bpe}
\title{BPE trainer}
\description{
BPE trainer

BPE trainer
}
\seealso{
Other trainer: 
\code{\link{tok_trainer}},
\code{\link{trainer_unigram}},
\code{\link{trainer_wordpiece}}
}
\concept{trainer}
\section{Super class}{
\code{\link[tok:tok_trainer]{tok::tok_trainer}} -> \code{tok_trainer_bpe}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_trainer_bpe-new}{\code{trainer_bpe$new()}}
\item \href{#method-tok_trainer_bpe-clone}{\code{trainer_bpe$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_trainer_bpe-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_trainer_bpe-new}{}}}
\subsection{Method \code{new()}}{
Constrcutor for the BPE trainer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{trainer_bpe$new(
  vocab_size = NULL,
  min_frequency = NULL,
  show_progress = NULL,
  special_tokens = NULL,
  limit_alphabet = NULL,
  initial_alphabet = NULL,
  continuing_subword_prefix = NULL,
  end_of_word_suffix = NULL,
  max_token_length = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vocab_size}}{The size of the final vocabulary, including all tokens and alphabet.
Default: \code{NULL}.}

\item{\code{min_frequency}}{The minimum frequency a pair should have in order to be merged.
Default: \code{NULL}.}

\item{\code{show_progress}}{Whether to show progress bars while training. Default: \code{TRUE}.}

\item{\code{special_tokens}}{A list of special tokens the model should be aware of.
Default: \code{NULL}.}

\item{\code{limit_alphabet}}{The maximum number of different characters to keep in the alphabet.
Default: \code{NULL}.}

\item{\code{initial_alphabet}}{A list of characters to include in the initial alphabet,
even if not seen in the training dataset. Default: \code{NULL}.}

\item{\code{continuing_subword_prefix}}{A prefix to be used for every subword that is not a beginning-of-word.
Default: \code{NULL}.}

\item{\code{end_of_word_suffix}}{A suffix to be used for every subword that is an end-of-word.
Default: \code{NULL}.}

\item{\code{max_token_length}}{Prevents creating tokens longer than the specified size.
Default: \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_trainer_bpe-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_trainer_bpe-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{trainer_bpe$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models.R
\name{tok_model}
\alias{tok_model}
\title{Generic class for tokenization models}
\description{
Generic class for tokenization models

Generic class for tokenization models
}
\seealso{
Other model: 
\code{\link{model_bpe}},
\code{\link{model_unigram}},
\code{\link{model_wordpiece}}
}
\concept{model}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{.model}}{stores the pointer to the model. internal}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_model-new}{\code{tok_model$new()}}
\item \href{#method-tok_model-clone}{\code{tok_model$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_model-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_model-new}{}}}
\subsection{Method \code{new()}}{
Initializes a genric abstract tokenizer model
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tok_model$new(model)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{model}}{Pointer to a tokenization model}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_model-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_model-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tok_model$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainers.R
\name{trainer_unigram}
\alias{trainer_unigram}
\title{Unigram tokenizer trainer}
\description{
Unigram tokenizer trainer

Unigram tokenizer trainer
}
\seealso{
Other trainer: 
\code{\link{tok_trainer}},
\code{\link{trainer_bpe}},
\code{\link{trainer_wordpiece}}
}
\concept{trainer}
\section{Super class}{
\code{\link[tok:tok_trainer]{tok::tok_trainer}} -> \code{tok_trainer_unigram}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_trainer_unigram-new}{\code{trainer_unigram$new()}}
\item \href{#method-tok_trainer_unigram-clone}{\code{trainer_unigram$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_trainer_unigram-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_trainer_unigram-new}{}}}
\subsection{Method \code{new()}}{
Constructor for the Unigram tokenizer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{trainer_unigram$new(
  vocab_size = 8000,
  show_progress = TRUE,
  special_tokens = NULL,
  shrinking_factor = 0.75,
  unk_token = NULL,
  max_piece_length = 16,
  n_sub_iterations = 2
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vocab_size}}{The size of the final vocabulary, including all tokens and alphabet.}

\item{\code{show_progress}}{Whether to show progress bars while training.}

\item{\code{special_tokens}}{A list of special tokens the model should be aware of.}

\item{\code{shrinking_factor}}{The shrinking factor used at each step of training
to prune the vocabulary.}

\item{\code{unk_token}}{The token used for out-of-vocabulary tokens.}

\item{\code{max_piece_length}}{The maximum length of a given token.}

\item{\code{n_sub_iterations}}{The number of iterations of the EM algorithm to perform
before pruning the vocabulary.}

\item{\code{initial_alphabet}}{A list of characters to include in the initial alphabet,
even if not seen in the training dataset. If the strings contain more than
one character, only the first one is kept.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_trainer_unigram-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_trainer_unigram-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{trainer_unigram$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models.R
\name{model_bpe}
\alias{model_bpe}
\title{BPE model}
\description{
BPE model

BPE model
}
\seealso{
Other model: 
\code{\link{model_unigram}},
\code{\link{model_wordpiece}},
\code{\link{tok_model}}
}
\concept{model}
\section{Super class}{
\code{\link[tok:tok_model]{tok::tok_model}} -> \code{tok_model_bpe}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_model_bpe-new}{\code{model_bpe$new()}}
\item \href{#method-tok_model_bpe-clone}{\code{model_bpe$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_model_bpe-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_model_bpe-new}{}}}
\subsection{Method \code{new()}}{
Initializes a BPE model
An implementation of the BPE (Byte-Pair Encoding) algorithm
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{model_bpe$new(
  vocab = NULL,
  merges = NULL,
  cache_capacity = NULL,
  dropout = NULL,
  unk_token = NULL,
  continuing_subword_prefix = NULL,
  end_of_word_suffix = NULL,
  fuse_unk = NULL,
  byte_fallback = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vocab}}{A named integer vector of string keys and their corresponding ids. Default: \code{NULL}}

\item{\code{merges}}{A list of pairs of tokens (\verb{[character, character]}). Default: \code{NULL}.}

\item{\code{cache_capacity}}{The number of words that the BPE cache can contain.
The cache speeds up the process by storing merge operation results. Default: \code{NULL.}}

\item{\code{dropout}}{A float between 0 and 1 representing the BPE dropout to use. Default: \code{NULL}}

\item{\code{unk_token}}{The unknown token to be used by the model. Default: `NULL```.}

\item{\code{continuing_subword_prefix}}{The prefix to attach to subword units that don’t
represent the beginning of a word. Default: \code{NULL}}

\item{\code{end_of_word_suffix}}{The suffix to attach to subword units that represent
the end of a word. Default: \code{NULL}}

\item{\code{fuse_unk}}{Whether to fuse any subsequent unknown tokens into a single one. Default: \code{NULL}.}

\item{\code{byte_fallback}}{Whether to use the spm byte-fallback trick. Default: \code{FALSE}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_model_bpe-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_model_bpe-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{model_bpe$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainers.R
\name{tok_trainer}
\alias{tok_trainer}
\title{Generic training class}
\description{
Generic training class

Generic training class
}
\seealso{
Other trainer: 
\code{\link{trainer_bpe}},
\code{\link{trainer_unigram}},
\code{\link{trainer_wordpiece}}
}
\concept{trainer}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{.trainer}}{a pointer to a raw trainer}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_trainer-new}{\code{tok_trainer$new()}}
\item \href{#method-tok_trainer-clone}{\code{tok_trainer$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_trainer-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_trainer-new}{}}}
\subsection{Method \code{new()}}{
Initializes a generic trainer from a raw trainer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tok_trainer$new(trainer)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{trainer}}{raw trainer (internal)}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_trainer-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_trainer-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{tok_trainer$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pre_tokenizers.R
\name{pre_tokenizer_whitespace}
\alias{pre_tokenizer_whitespace}
\title{This pre-tokenizer simply splits using the following regex: \verb{\\w+|[^\\w\\s]+}}
\description{
This pre-tokenizer simply splits using the following regex: \verb{\\w+|[^\\w\\s]+}

This pre-tokenizer simply splits using the following regex: \verb{\\w+|[^\\w\\s]+}
}
\seealso{
Other pre_tokenizer: 
\code{\link{pre_tokenizer}},
\code{\link{pre_tokenizer_byte_level}}
}
\concept{pre_tokenizer}
\section{Super class}{
\code{tok::tok_pre_tokenizer} -> \code{tok_pre_tokenizer_whitespace}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_pre_tokenizer_whitespace-new}{\code{pre_tokenizer_whitespace$new()}}
\item \href{#method-tok_pre_tokenizer_whitespace-clone}{\code{pre_tokenizer_whitespace$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_pre_tokenizer_whitespace-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_pre_tokenizer_whitespace-new}{}}}
\subsection{Method \code{new()}}{
Initializes the whistespace tokenizer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{pre_tokenizer_whitespace$new()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_pre_tokenizer_whitespace-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_pre_tokenizer_whitespace-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{pre_tokenizer_whitespace$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainers.R
\name{trainer_wordpiece}
\alias{trainer_wordpiece}
\title{WordPiece tokenizer trainer}
\description{
WordPiece tokenizer trainer

WordPiece tokenizer trainer
}
\seealso{
Other trainer: 
\code{\link{tok_trainer}},
\code{\link{trainer_bpe}},
\code{\link{trainer_unigram}}
}
\concept{trainer}
\section{Super class}{
\code{\link[tok:tok_trainer]{tok::tok_trainer}} -> \code{tok_trainer_wordpiece}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_trainer_wordpiece-new}{\code{trainer_wordpiece$new()}}
\item \href{#method-tok_trainer_wordpiece-clone}{\code{trainer_wordpiece$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_trainer_wordpiece-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_trainer_wordpiece-new}{}}}
\subsection{Method \code{new()}}{
Constructor for the WordPiece tokenizer trainer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{trainer_wordpiece$new(
  vocab_size = 30000,
  min_frequency = 0,
  show_progress = FALSE,
  special_tokens = NULL,
  limit_alphabet = NULL,
  initial_alphabet = NULL,
  continuing_subword_prefix = "##",
  end_of_word_suffix = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vocab_size}}{The size of the final vocabulary, including all tokens and alphabet.
Default: \code{NULL}.}

\item{\code{min_frequency}}{The minimum frequency a pair should have in order to be merged.
Default: \code{NULL}.}

\item{\code{show_progress}}{Whether to show progress bars while training. Default: \code{TRUE}.}

\item{\code{special_tokens}}{A list of special tokens the model should be aware of.
Default: \code{NULL}.}

\item{\code{limit_alphabet}}{The maximum number of different characters to keep in the alphabet.
Default: \code{NULL}.}

\item{\code{initial_alphabet}}{A list of characters to include in the initial alphabet,
even if not seen in the training dataset. If the strings contain more than
one character, only the first one is kept. Default: \code{NULL}.}

\item{\code{continuing_subword_prefix}}{A prefix to be used for every subword that is not a beginning-of-word.
Default: \code{NULL}.}

\item{\code{end_of_word_suffix}}{A suffix to be used for every subword that is an end-of-word.
Default: \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_trainer_wordpiece-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_trainer_wordpiece-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{trainer_wordpiece$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decoders.R
\name{decoder_byte_level}
\alias{decoder_byte_level}
\title{Byte level decoder}
\description{
Byte level decoder

Byte level decoder
}
\details{
This decoder is to be used with the \link{pre_tokenizer_byte_level}.
}
\seealso{
Other decoders: 
\code{\link{tok_decoder}}
}
\concept{decoders}
\section{Super class}{
\code{\link[tok:tok_decoder]{tok::tok_decoder}} -> \code{tok_decoder_byte_level}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_decoder_byte_level-new}{\code{decoder_byte_level$new()}}
\item \href{#method-tok_decoder_byte_level-clone}{\code{decoder_byte_level$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_decoder_byte_level-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_decoder_byte_level-new}{}}}
\subsection{Method \code{new()}}{
Initializes a byte level decoder
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{decoder_byte_level$new()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_decoder_byte_level-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_decoder_byte_level-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{decoder_byte_level$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/encoding.R
\name{encoding}
\alias{encoding}
\title{Encoding}
\value{
An encoding object containing encoding information such as attention masks
and token ids.
}
\description{
Represents the output of a \link{tokenizer}.
}
\examples{
withr::with_envvar(c(HUGGINGFACE_HUB_CACHE = tempdir()), {
try({
tok <- tokenizer$from_pretrained("gpt2")
encoding <- tok$encode("Hello world")
encoding
})
})
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{.encoding}}{The underlying implementation pointer.}
}
\if{html}{\out{</div>}}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{ids}}{The IDs are the main input to a Language Model. They are the
token indices, the numerical representations that a LM understands.}

\item{\code{attention_mask}}{The attention mask used as input for transformers models.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-tok_encoding-new}{\code{encoding$new()}}
\item \href{#method-tok_encoding-clone}{\code{encoding$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_encoding-new"></a>}}
\if{latex}{\out{\hypertarget{method-tok_encoding-new}{}}}
\subsection{Method \code{new()}}{
Initializes an encoding object (Not to use directly)
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{encoding$new(encoding)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{encoding}}{an encoding implementation object}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-tok_encoding-clone"></a>}}
\if{latex}{\out{\hypertarget{method-tok_encoding-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{encoding$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}
